//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, Oren Avissar
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in
//   the documentation and/or other materials provided with the
//   distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// sleep.c: implementation of the sleep module.
//
//////////////////////////////////////////////////////////////////////////////

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include "sleep.h"


//////// Internal Global Variable Declarations ////////

    //millisecond counter (incremented every ms)
static volatile uint16_t _counterms = 0;

//////// Internal Function Declarations ////////

    //ISR for Timer/Counter0 Overflow
void SIG_OVERFLOW0 (void) __attribute__ ((signal));


//////////////////////////////////////////////////////////////////////
// Timer 0 ISR
//////////////////////////////////////////////////////////////////////

    //set the interrupt for Timer/Counter0 Overflow
void SIG_OVERFLOW0 (void)
{

        //reload the timer value
    TCNT0 = TIMER_TCNT0_VALUE;

        //increment the millisecond counter
    _counterms++;
}

//////////////////////////////////////////////////////////////////////
// sleep functions
//////////////////////////////////////////////////////////////////////

void sleep(uint16_t timems)
{

        //initialize the Timer/Counter0 control register
    TCCR0 = (1 << CS01) | (1 << PSR0);  //set prescaler to clk/8

        //initialize the timer counter (creates a 1 ms delay)
    TCNT0 = TIMER_TCNT0_VALUE;

        //set the Timer/Counter0 Overflow Interrupt Enable
    TIMSK |= (1 << TOIE0);

        //reset the counter
    _counterms = 0;

        //wait until the specified time has passed
    while (_counterms < timems);

        //stop the Timer/Counter0
    TCCR0 = 0;
}
